from xml.dom.minidom import getDOMImplementation
from xml.etree import ElementTree

_xmldom_impl = getDOMImplementation()

def dict_to_xml(valueDict):
    doc = _xmldom_impl.createDocument(None, 'xml', None)
    topElement = doc.documentElement
    for (key, value) in valueDict.items():
        element = doc.createElement(key)
        element.appendChild(doc.createTextNode(str(value)))
        topElement.appendChild(element)
    return topElement.toxml()

def xml_to_dict(xmlContent):
    result = {}
    root = ElementTree.fromstring(xmlContent)
    for child in root:
        result[child.tag] = child.text
    return result


# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-05-17 13:56
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='AliPayOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('out_trade_no', models.CharField(db_index=True, editable=False, max_length=32, verbose_name='商户订单号')),
                ('subject', models.CharField(editable=False, max_length=128, verbose_name='商品名称')),
                ('body', models.CharField(editable=False, max_length=512, verbose_name='商品详情')),
                ('total_fee', models.DecimalField(decimal_places=2, editable=False, max_digits=6, verbose_name='总金额(单位:元)')),
                ('it_b_pay', models.CharField(editable=False, max_length=19, verbose_name='交易有效期')),
                ('interface_data', models.TextField(editable=False, max_length=500, verbose_name='接口数据')),
                ('date_create', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
            ],
            options={
                'verbose_name_plural': '支付宝订单',
                'verbose_name': '支付宝订单',
            },
        ),
        migrations.CreateModel(
            name='OrderItem',
            fields=[
                ('orderno', models.CharField(editable=False, max_length=50, primary_key=True, serialize=False, verbose_name='订单号')),
                ('user', models.CharField(blank=True, max_length=50, null=True, verbose_name='用户标识')),
                ('product_desc', models.CharField(max_length=128, verbose_name='商品描述')),
                ('product_detail', models.TextField(max_length=1000, verbose_name='商品详情')),
                ('fee', models.DecimalField(decimal_places=0, max_digits=6, verbose_name='金额(单位:分)')),
                ('attach', models.CharField(blank=True, max_length=127, null=True, verbose_name='附加数据')),
                ('dt_start', models.DateTimeField(editable=False, verbose_name='交易开始时间')),
                ('dt_end', models.DateTimeField(editable=False, verbose_name='交易失效时间')),
                ('dt_pay', models.DateTimeField(blank=True, editable=False, null=True, verbose_name='付款时间')),
                ('paied', models.BooleanField(default=False, editable=False, verbose_name='已收款')),
                ('lapsed', models.BooleanField(default=False, editable=False, verbose_name='已失效')),
                ('payway', models.CharField(choices=[('WEIXIN', '微信支付'), ('ALI', '支付宝支付')], default='WEIXIN', max_length=10, verbose_name='支付方式')),
                ('date_create', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('date_update', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
            ],
            options={
                'verbose_name_plural': '付款单',
                'verbose_name': '付款单',
            },
        ),
        migrations.CreateModel(
            name='WeiXinOrder',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('appid', models.CharField(editable=False, max_length=32, verbose_name='公众账号ID')),
                ('mch_id', models.CharField(editable=False, max_length=32, verbose_name='商户号')),
                ('body', models.CharField(editable=False, max_length=128, verbose_name='商品描述')),
                ('attach', models.CharField(blank=True, editable=False, max_length=127, null=True, verbose_name='附加数据')),
                ('out_trade_no', models.CharField(db_index=True, editable=False, max_length=32, verbose_name='商户订单号')),
                ('fee_type', models.CharField(editable=False, max_length=16, verbose_name='货币类型')),
                ('total_fee', models.SmallIntegerField(editable=False, verbose_name='总金额')),
                ('spbill_create_ip', models.CharField(editable=False, max_length=16, verbose_name='终端IP')),
                ('time_start', models.CharField(editable=False, max_length=14, verbose_name='交易起始时间')),
                ('time_expire', models.CharField(editable=False, max_length=14, verbose_name='交易结束时间')),
                ('notify_url', models.CharField(editable=False, max_length=256, verbose_name='通知地址')),
                ('trade_type', models.CharField(editable=False, max_length=16, verbose_name='交易类型')),
            ],
            options={
                'verbose_name_plural': '微信统一订单',
                'verbose_name': '微信统一订单',
            },
        ),
        migrations.CreateModel(
            name='AliPayResult',
            fields=[
                ('order', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='pay_result', serialize=False, to='openunipay.AliPayOrder')),
                ('notify_time', models.CharField(blank=True, editable=False, max_length=19, null=True, verbose_name='通知时间')),
                ('notify_type', models.CharField(blank=True, editable=False, max_length=50, null=True, verbose_name='通知类型')),
                ('notify_id', models.CharField(blank=True, editable=False, max_length=50, null=True, verbose_name='通知校验ID')),
                ('out_trade_no', models.CharField(blank=True, editable=False, max_length=32, null=True, verbose_name='商户订单号')),
                ('subject', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='商品名称')),
                ('trade_no', models.CharField(blank=True, editable=False, max_length=64, null=True, verbose_name='支付宝交易号')),
                ('trade_status', models.CharField(blank=True, editable=False, max_length=16, null=True, verbose_name='交易状态')),
                ('seller_id', models.CharField(blank=True, editable=False, max_length=30, null=True, verbose_name='卖家支付宝用户号')),
                ('seller_email', models.CharField(blank=True, editable=False, max_length=100, null=True, verbose_name='卖家支付宝账号')),
                ('buyer_id', models.CharField(blank=True, editable=False, max_length=30, null=True, verbose_name='买家支付宝用户号')),
                ('buyer_email', models.CharField(blank=True, editable=False, max_length=100, null=True, verbose_name='买家支付宝账号  ')),
                ('total_fee', models.DecimalField(blank=True, decimal_places=2, editable=False, max_digits=6, null=True, verbose_name='总金额(单位:元)')),
            ],
        ),
        migrations.CreateModel(
            name='WeiXinPayResult',
            fields=[
                ('order', models.OneToOneField(editable=False, on_delete=django.db.models.deletion.CASCADE, primary_key=True, related_name='pay_result', serialize=False, to='openunipay.WeiXinOrder')),
                ('prepayid', models.CharField(blank=True, db_index=True, editable=False, max_length=64, null=True, verbose_name='预支付交易会话标识')),
                ('openid', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='用户标识(openId)')),
                ('bank_type', models.CharField(blank=True, editable=False, max_length=16, null=True, verbose_name='付款银行')),
                ('total_fee', models.SmallIntegerField(blank=True, editable=False, null=True, verbose_name='总金额')),
                ('attach', models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='商户附加数据')),
                ('tradestate', models.CharField(blank=True, editable=False, max_length=32, null=True, verbose_name='交易状态')),
                ('tradestatedesc', models.CharField(blank=True, editable=False, max_length=256, null=True, verbose_name='交易状态描述')),
            ],
        ),
    ]



# -*- coding: utf-8 -*-
import requests
import logging
from urllib import parse
from django.conf import settings
from openunipay.weixin_pay.exceptions import APIError
from openunipay.weixin_pay.models import WeiXinOrder, WeiXinPayResult, WeiXinQRPayRecord
from openunipay.weixin_pay.security import sign
from openunipay.weixin_pay import xml_helper
from openunipay.util import random_helper
from openunipay.paygateway import PayResult
from openunipay import exceptions

CODE_SUCC = 'SUCCESS'
TRADE_STATE_SUCC = ('SUCCESS', )
TRADE_STATE_LAPSED = ('CLOSED', 'REVOKED')

_logger = logging.getLogger('openunipay.weixin')


############### 统一下单 #######################
def create_order(weixinOrderObj):
    assert isinstance(weixinOrderObj, WeiXinOrder)
    _logger.info("creating order")
    payResultObj = WeiXinPayResult.objects.create(order=weixinOrderObj)
    url = 'https://api.mch.weixin.qq.com/pay/unifiedorder'
    data = weixinOrderObj.to_xml().encode()
    _logger.info("request data is:{}".format(data))
    r = requests.post(url, data=data, headers={'Content-Type': 'application/xml'}, verify=False)

    def handle_response(responseData):
        payResultObj.prepayid = responseData['prepay_id']
        payResultObj.save()
        _logger.info("order created. prepayid is {}".format(payResultObj.prepayid))
        return responseData['prepay_id']

    return __handle_weixin_api_xml_response(r, handle_response)


############# 处理微信支付 异步通知 #############
def process_notify(notifyContent):
    responseData = xml_helper.xml_to_dict(notifyContent)
    # check sign
    if responseData['return_code'] == CODE_SUCC:
        __verify_response_data(responseData)
        result = _process_order_result(responseData)
        return result
    else:
        return None


def query_order(orderNo):
    weixinOrderObj = WeiXinOrder.objects.get(out_trade_no=orderNo)

    result = _compose_pay_result(orderNo, weixinOrderObj.pay_result.tradestate)
    if result.Succ or result.Lapsed:
        return result

    valueDict = {'appid': weixinOrderObj.appid, 'mch_id': weixinOrderObj.mch_id, 'out_trade_no': weixinOrderObj.out_trade_no, 'nonce_str': random_helper.generate_nonce_str(23)}
    valueDict['sign'] = sign(valueDict)
    data = xml_helper.dict_to_xml(valueDict)

    url = 'https://api.mch.weixin.qq.com/pay/orderquery'
    r = requests.post(url, data=data, headers={'Content-Type': 'application/xml'}, verify=False)

    return __handle_weixin_api_xml_response(r, _process_order_result)


def _process_order_result(responseData):
    # save data
    weixinOrderObj = WeiXinOrder.objects.get(out_trade_no=responseData['out_trade_no'])
    payResultObj = weixinOrderObj.pay_result
    payResultObj.openid = responseData.get('openid')
    payResultObj.bank_type = responseData.get('bank_type')
    payResultObj.total_fee = responseData.get('total_fee')
    payResultObj.attach = responseData.get('attach')
    payResultObj.tradestate = responseData.get('trade_state', 'SUCCESS')
    payResultObj.tradestatedesc = responseData.get('trade_state_desc')
    payResultObj.save()
    result = _compose_pay_result(responseData['out_trade_no'], payResultObj.tradestate)
    return result


def _compose_pay_result(orderNo, tradestate):
    result = PayResult(orderNo)
    result.succ = tradestate in TRADE_STATE_SUCC
    result.lapsed = tradestate in TRADE_STATE_LAPSED
    return result


############ 扫码支付 ##################
def process_qr_pay_notify(notifyContent):
    '''
    @summary: 处理微信扫码支付异步通知
    '''
    responseData = xml_helper.xml_to_dict(notifyContent)
    # check sign
    __verify_response_data(responseData)
    # process data
    qrPayRecord = WeiXinQRPayRecord.objects.create(
        appid=responseData.get('appid'), mch_id=responseData.get('mch_id'), openid=responseData.get('openid'), product_id=responseData.get('product_id'))
    return {'product_id': qrPayRecord.product_id, 'openid': qrPayRecord.openid, 'nonce_str': responseData.get('nonce_str')}


############ 转换短连接 ##################
def request_shorten_url(url):
    _logger.info("get shorten url")
    data = {
        'appid': settings.WEIXIN['app_id'],
        'mch_id': settings.WEIXIN['mch_id'],
        'long_url': url,
        'nonce_str': random_helper.generate_nonce_str(23),
    }
    data['sign'] = sign(data)
    requestBody = xml_helper.dict_to_xml(data)
    _logger.info("request data is:{}".format(requestBody))

    url = 'https://api.mch.weixin.qq.com/tools/shorturl'
    r = requests.post(url, data=requestBody, headers={'Content-Type': 'application/xml'}, verify=False)
    return __handle_weixin_api_xml_response(r, lambda responseData: responseData['short_url'])


########
def __verify_response_data(responseData):
    '''
    @summary: 使用签名验证微信服务器的返回数据
    @param responseData:返回数据的Dict,里面需包含sign字段. 验证成功后sign字段会被删除
    @return 如果验证失败会抛出异常
    '''
    signStr = responseData['sign']
    del responseData['sign']
    signCheck = sign(responseData)
    if signStr != signCheck:
        _logger.error('received untrusted data')
        raise exceptions.InsecureDataError()


def __handle_weixin_api_xml_response(r, func):
    '''
    @summary: 处理微信API返回结果
    @param func:数据处理function, 会在请求成功并且验证成功的时候调用
    @return 如果验证失败会抛出异常
    '''
    r.encoding = 'utf-8'
    _logger.info("response body is:{}".format(r.text))
    if r.status_code == 200:
        responseData = xml_helper.xml_to_dict(r.text)
        if responseData['return_code'] == CODE_SUCC:
            # check sign
            __verify_response_data(responseData)
            if responseData['result_code'] == CODE_SUCC:
                return func(responseData)
            else:
                raise APIError(responseData['err_code'])
        else:
            raise APIError('data sign verfied failed')
    else:
        raise APIError('status from weixin is not 200')



# -*- coding: utf-8 -*-
from django.db import models
from django.conf import settings
from urllib.parse import quote_plus
from openunipay.ali_pay import security

_SERVICE = 'mobile.securitypay.pay'
_CHARSET = 'utf-8'
_SIGN_TYPE = 'RSA'
_PAYMENT_TYPE = '1'

_ALIPAY_ORDER_FIELD = ('out_trade_no', 'subject', 'body', 'total_fee', 'it_b_pay',)

class AliPayOrder(models.Model):
    out_trade_no = models.CharField(verbose_name='商户订单号', max_length=32, db_index=True, editable=False)
    subject = models.CharField(verbose_name='商品名称', max_length=128, editable=False)
    body = models.CharField(verbose_name='商品详情', max_length=512, editable=False)
    total_fee = models.DecimalField(verbose_name='总金额(单位:元)', max_digits=6, decimal_places=2, editable=False)
    it_b_pay = models.CharField(verbose_name='交易有效期', max_length=19, editable=False)
    date_create = models.DateTimeField(verbose_name=u'创建时间', auto_now_add=True)
    
    class Meta:
        verbose_name = u'支付宝订单'
        verbose_name_plural = u'支付宝订单'
        
    def __str__(self):
        return self.out_trade_no
        
    def compose_interface_data(self):
        # sign data
        data = self._compose_data()
        return '{}&sign_type="RSA"&sign="{}"'.format(data, quote_plus(security.sign(data)))
    
    def _compose_data(self):
        valueDict = {item:getattr(self, item) 
                for item in _ALIPAY_ORDER_FIELD 
                if getattr(self, item)}
        valueDict['service'] = _SERVICE
        valueDict['_input_charset'] = _CHARSET
        valueDict['payment_type'] = _PAYMENT_TYPE
        valueDict['partner'] = settings.ALIPAY['partner']
        valueDict['seller_id'] = settings.ALIPAY['seller_id']
        valueDict['notify_url'] = settings.ALIPAY['notify_url']
        
        temp = []
        for key in valueDict:
            if not valueDict[key]:
                continue
            temp.append('{}="{}"'.format(key, valueDict[key]))
        tempStr = '&'.join(temp)
        return tempStr
    
class AliPayResult(models.Model):
    order = models.OneToOneField(AliPayOrder,
                                 on_delete=models.CASCADE,
                                 primary_key=True,
                                 editable=False,
                                 related_name='pay_result')
    notify_time = models.CharField(verbose_name=u'通知时间', null=True, blank=True, max_length=19, editable=False)
    notify_type = models.CharField(verbose_name=u'通知类型', null=True, blank=True, max_length=50, editable=False)
    notify_id = models.CharField(verbose_name=u'通知校验ID', null=True, blank=True, max_length=50, editable=False)
    out_trade_no = models.CharField(verbose_name=u'商户订单号', null=True, blank=True, max_length=32, editable=False)
    subject = models.CharField(verbose_name=u'商品名称', null=True, blank=True, max_length=128, editable=False)
    trade_no = models.CharField(verbose_name=u'支付宝交易号', null=True, blank=True, max_length=64, editable=False)
    trade_status = models.CharField(verbose_name=u'交易状态', null=True, blank=True, max_length=16, editable=False)
    seller_id = models.CharField(verbose_name=u'卖家支付宝用户号', null=True, blank=True, max_length=30, editable=False)
    seller_email = models.CharField(verbose_name=u'卖家支付宝账号', null=True, blank=True, max_length=100, editable=False)
    buyer_id = models.CharField(verbose_name=u'买家支付宝用户号', null=True, blank=True, max_length=30, editable=False)
    buyer_email = models.CharField(verbose_name=u'买家支付宝账号  ', null=True, blank=True, max_length=100, editable=False)
    total_fee = models.DecimalField(verbose_name=u'总金额(单位:元)', null=True, blank=True, max_digits=6, decimal_places=2, editable=False)
    
    
    def __str__(self):
        fieldsList = AliPayResult._meta.get_fields()
        temp = []
        for field in fieldsList:
            if not field.auto_created:
                temp.append('{}:{}'.format(field.verbose_name, getattr(self, field.attname)))
        return ','.join(temp) 
        



# -*- coding: utf-8 -*-
from django.contrib import admin
from .models import AliPayOrder
from import_export import resources
from import_export.admin import ImportExportModelAdmin

class AliPayOrderResource(resources.ModelResource):
    class Meta:
        model = AliPayOrder
        import_id_fields = ('id',)

class AliPayOrderAdmin(ImportExportModelAdmin):
    resource_class = AliPayOrderResource
    # list page
    list_display = ('date_create', 'out_trade_no', 'subject', 'body', 'total_fee', 'it_b_pay', 'get_pay_result',)
    ordering = ('-date_create',)
    search_fields = ['=out_trade_no', ]
    
    def get_pay_result(self, obj):
        if hasattr(obj, 'pay_result'):
            return obj.pay_result
        else:
            return '-'
    get_pay_result.short_description = '支付结果'
    
    
admin.site.register(AliPayOrder, AliPayOrderAdmin)



from . import PayGateway
from django.db import transaction
from django.conf import settings
from openunipay.util import random_helper, datetime
from openunipay.weixin_pay.models import WeiXinOrder, WeiXinQRPayEntity
from openunipay.weixin_pay import weixin_pay_lib, security, xml_helper
from openunipay.models import OrderItem, Product, PAY_WAY_WEIXIN


class WeiXinPayGateway(PayGateway):
    @transaction.atomic
    def create_order(self, orderItemObj, clientIp, **kwargs):
        weixinOrderObj = WeiXinOrder()
        weixinOrderObj.appid = settings.WEIXIN['app_id']
        weixinOrderObj.mch_id = settings.WEIXIN['mch_id']
        weixinOrderObj.body = orderItemObj.product_desc
        weixinOrderObj.attach = orderItemObj.attach
        weixinOrderObj.out_trade_no = orderItemObj.orderno
        weixinOrderObj.total_fee = orderItemObj.fee
        weixinOrderObj.spbill_create_ip = clientIp
        weixinOrderObj.time_start = orderItemObj.dt_start.strftime("%Y%m%d%H%M%S")
        weixinOrderObj.time_expire = orderItemObj.dt_end.strftime("%Y%m%d%H%M%S")
        weixinOrderObj.notify_url = settings.WEIXIN['mch_notify_url']
        weixinOrderObj.trade_type = kwargs.get('trade_type', 'APP')
        weixinOrderObj.openid = kwargs.get('openid')
        weixinOrderObj.save()
        prepayid = weixin_pay_lib.create_order(weixinOrderObj)
        data = {'appid': settings.WEIXIN['app_id'],
                'partnerid': settings.WEIXIN['mch_id'],
                'prepayid': prepayid,
                'package': 'Sign=WXPay',
                'noncestr': random_helper.generate_nonce_str(23),
                'timestamp': str(datetime.get_unix_timestamp())}
        data['sign'] = security.sign(data)
        return data

    @transaction.atomic
    def process_notify(self, requestContent):
        return weixin_pay_lib.process_notify(requestContent)

    @transaction.atomic
    def query_order(self, orderNo):
        return weixin_pay_lib.query_order(orderNo)

    @transaction.atomic
    def generate_qr_pay_url(self, product_id):
        qrPayEntiry = WeiXinQRPayEntity()
        qrPayEntiry.appid = settings.WEIXIN['app_id']
        qrPayEntiry.mch_id = settings.WEIXIN['mch_id']
        qrPayEntiry.time_stamp = str(datetime.get_unix_timestamp())
        qrPayEntiry.product_id = product_id
        qrPayEntiry.save()
        url = weixin_pay_lib.request_shorten_url(qrPayEntiry.to_raw_rul())
        return url

    @transaction.atomic
    def process_qr_pay_notify(self, requestContent):
        notifyObj = weixin_pay_lib.process_qr_pay_notify(requestContent)

        productObj = Product.objects.get(productid=notifyObj['product_id'])

        orderItemObj = OrderItem()
        orderItemObj.orderno = self._generate_qr_orderno(productObj.productid)
        orderItemObj.user = notifyObj['openid']
        orderItemObj.product_desc = productObj.product_desc
        orderItemObj.product_detail = productObj.product_detail
        orderItemObj.fee = productObj.fee
        orderItemObj.payway = PAY_WAY_WEIXIN
        orderItemObj.attach = None
        orderItemObj.initial_orlder(1440)
        orderItemObj.save()

        weixinOrderItem = self.create_order(orderItemObj, settings.WEIXIN['clientIp'], trade_type='NATIVE', openid=notifyObj['openid'])

        data = {
            'return_code': 'SUCCESS',
            'result_code': 'SUCCESS',
            'appid': settings.WEIXIN['app_id'],
            'mch_id': settings.WEIXIN['mch_id'],
            'prepay_id': weixinOrderItem['prepayid'],
            'nonce_str': notifyObj['nonce_str'],
        }
        data['sign'] = security.sign(data)
        return xml_helper.dict_to_xml(data)

    def _generate_qr_orderno(self, productid):
        return 'WXQR-{}-{}'.format(productid, datetime.get_unix_timestamp())



import hashlib
from django.conf import settings

def sign(valueDict):
    temp = []
    for key in sorted(valueDict):
        if not valueDict[key]:
            continue
        temp.append('{}={}'.format(key, valueDict[key]))
    temp.append('key=' + settings.WEIXIN['mch_seckey'])
    tempStr = '&'.join(temp)
    m = hashlib.md5()
    m.update(tempStr.encode())
    return m.hexdigest().upper()



# -*- coding: utf-8 -*-

from django.db import models
from openunipay.ali_pay.models import *
from openunipay.weixin_pay.models import *
from openunipay.util import datetime
from datetime import timedelta

PAY_WAY_WEIXIN = 'WEIXIN'
PAY_WAY_ALI = 'ALI'
PAY_WAY = ((PAY_WAY_WEIXIN, u'微信支付'),
           (PAY_WAY_ALI, u'支付宝支付'), )


class OrderItem(models.Model):
    orderno = models.CharField(verbose_name=u'订单号', max_length=50, primary_key=True, editable=False)
    user = models.CharField(verbose_name=u'用户标识', max_length=50, null=True, blank=True)
    product_desc = models.CharField(verbose_name=u'商品描述', max_length=128, null=False, blank=False)
    product_detail = models.TextField(verbose_name=u'商品详情', max_length=1000, null=False, blank=False)
    fee = models.DecimalField(verbose_name=u'金额(单位:分)', max_digits=6, decimal_places=0, null=False, blank=False)
    attach = models.CharField(verbose_name=u'附加数据', max_length=127, null=True, blank=True)
    dt_start = models.DateTimeField(verbose_name=u'交易开始时间', null=False, blank=False, editable=False)
    dt_end = models.DateTimeField(verbose_name=u'交易失效时间', null=False, blank=False, editable=False)
    dt_pay = models.DateTimeField(verbose_name=u'付款时间', null=True, blank=True, editable=False)
    paied = models.BooleanField(verbose_name=u'已收款', null=False, blank=False, default=False, editable=False)
    lapsed = models.BooleanField(verbose_name=u'已失效', null=False, blank=False, default=False, editable=False)
    payway = models.CharField(verbose_name=u'支付方式', max_length=10, null=False, blank=False, choices=PAY_WAY, default=PAY_WAY[0][0])
    date_create = models.DateTimeField(verbose_name=u'创建时间', auto_now_add=True)
    date_update = models.DateTimeField(verbose_name=u'修改时间', auto_now=True)

    class Meta:
        verbose_name = '付款单'
        verbose_name_plural = '付款单'

    def __str__(self):
        return self.orderno

    def _set_expire_time(self, expire):
        self.dt_start = datetime.local_now()
        self.dt_end = self.dt_start + timedelta(minutes=expire)

    def initial_orlder(self, expire):
        self._set_expire_time(expire)


class Product(models.Model):
    productid = models.CharField(verbose_name=u'商品ID', max_length=50, primary_key=True)
    product_desc = models.CharField(verbose_name=u'商品描述', max_length=128, null=False, blank=False)
    product_detail = models.TextField(verbose_name=u'商品详情', max_length=1000, null=False, blank=False)
    fee = models.DecimalField(verbose_name=u'金额(单位:分)', max_digits=6, decimal_places=0, null=False, blank=False)
    weinxin_qrurl = models.CharField(verbose_name=u'微信扫码支付URL', max_length=500, null=True, blank=True)
    date_create = models.DateTimeField(verbose_name=u'创建时间', auto_now_add=True)
    date_update = models.DateTimeField(verbose_name=u'修改时间', auto_now=True)

    class Meta:
        verbose_name = '商品'
        verbose_name_plural = '商品'



# -*- coding: utf-8 -*-
from .models import AliPayOrder, AliPayResult
from openunipay.paygateway import PayResult
from openunipay.ali_pay import logger, security
from openunipay import exceptions

TRADE_STATUS_SUCC = ('TRADE_SUCCESS', 'TRADE_FINISHED',)
TRADE_STATUS_LAPSED = ('TRADE_CLOSED',)

def create_order(orderObj):
    assert isinstance(orderObj, AliPayOrder)
    AliPayResult.objects.create(order=orderObj)
    
def process_notify(request):
    valueDict = request.POST.dict()
    if security.verify_ali_data(valueDict):
        # save data
        orderObj = AliPayOrder.objects.get(out_trade_no=valueDict['out_trade_no'])
        payResultObj = orderObj.pay_result
        payResultObj.out_trade_no = valueDict.get('out_trade_no')
        payResultObj.notify_time = valueDict.get('notify_time')
        payResultObj.notify_type = valueDict.get('notify_type')
        payResultObj.notify_id = valueDict.get('notify_id')
        payResultObj.subject = valueDict.get('subject')
        payResultObj.trade_no = valueDict.get('trade_no')
        payResultObj.trade_status = valueDict.get('trade_status')
        payResultObj.seller_id = valueDict.get('seller_id')
        payResultObj.seller_email = valueDict.get('seller_email')
        payResultObj.buyer_id = valueDict.get('buyer_id')
        payResultObj.buyer_email = valueDict.get('buyer_email')
        payResultObj.total_fee = valueDict.get('total_fee')
        payResultObj.save()
        result = _compose_pay_result(payResultObj.out_trade_no, payResultObj.trade_status)
        return result     
    else:
        logger.error('received unverified notification:{}'.format(valueDict))
        raise exceptions.InsecureDataError()

def query_order(orderNo):
    orderObj = AliPayOrder.objects.get(out_trade_no=orderNo)
    result = _compose_pay_result(orderObj.out_trade_no, orderObj.pay_result.trade_status)
    return result 

def _compose_pay_result(orderNo, trade_status):
    result = PayResult(orderNo)
    result.succ = trade_status in TRADE_STATUS_SUCC
    result.lapsed = trade_status in TRADE_STATUS_LAPSED
    return result      



# -*- coding: utf-8 -*-
from django.db import models
from openunipay.util import random_helper
from .security import sign
from . import xml_helper
from urllib import parse

_ORDER_NONCE_POPULATION = '123457890ABCDEFGHIJKLMNOPQRSTUVWXYZ'


class WeiXinQRPayEntity(models.Model):
    appid = models.CharField(verbose_name=u'公众账号ID', max_length=32, editable=False)
    mch_id = models.CharField(verbose_name=u'商户号', max_length=32, editable=False)
    time_stamp = models.CharField(verbose_name=u'时间戳', max_length=10, editable=False)
    product_id = models.CharField(verbose_name=u'商品ID', max_length=32, editable=False)

    class Meta:
        verbose_name = u'微信扫码支付-二维码URL'
        verbose_name_plural = u'微信扫码支付-二维码URL'

    def _get_vlaue_dict(self):
        fieldsList = WeiXinQRPayEntity._meta.get_fields()
        return {item.attname: getattr(self, item.attname) for item in fieldsList if not item.auto_created and getattr(self, item.attname)}

    def to_url(self):
        # sign data
        valueDict = self._get_vlaue_dict()
        valueDict['nonce_str'] = random_helper.generate_nonce_str(23)
        valueDict['sign'] = sign(valueDict)
        urlparameer = parse.urlencode(valueDict)
        return 'weixin://wxpay/bizpayurl?{}'.format(urlparameer)

    def to_raw_rul(self):
        valueDict = self._get_vlaue_dict()
        valueDict['nonce_str'] = random_helper.generate_nonce_str(23)
        valueDict['sign'] = sign(valueDict)
        temp = ['{}={}'.format(k, v) for k, v in valueDict.items()]
        return 'weixin://wxpay/bizpayurl?{}'.format('&'.join(temp))


class WeiXinQRPayRecord(models.Model):
    appid = models.CharField(verbose_name=u'公众账号ID', max_length=32, editable=False)
    mch_id = models.CharField(verbose_name=u'商户号', max_length=32, editable=False)
    openid = models.CharField(verbose_name=u'用户标识', max_length=128, editable=False)
    product_id = models.CharField(verbose_name=u'商品ID', max_length=32, editable=False)

    class Meta:
        verbose_name = u'微信扫码支付-扫码纪录'
        verbose_name_plural = u'微信扫码支付-扫码纪录'


class WeiXinOrder(models.Model):
    appid = models.CharField(verbose_name=u'公众账号ID', max_length=32, editable=False)
    mch_id = models.CharField(verbose_name=u'商户号', max_length=32, editable=False)
    body = models.CharField(verbose_name=u'商品描述', max_length=128, editable=False)
    attach = models.CharField(verbose_name=u'附加数据', max_length=127, null=True, blank=True, editable=False)
    out_trade_no = models.CharField(verbose_name=u'商户订单号', max_length=32, db_index=True, editable=False)
    fee_type = models.CharField(verbose_name=u'货币类型', max_length=16, editable=False)
    total_fee = models.PositiveIntegerField(verbose_name=u'总金额', editable=False)
    spbill_create_ip = models.CharField(verbose_name=u'终端IP', max_length=16, editable=False)
    time_start = models.CharField(verbose_name=u'交易起始时间', max_length=14, editable=False)
    time_expire = models.CharField(verbose_name=u'交易结束时间', max_length=14, editable=False)
    notify_url = models.CharField(verbose_name=u'通知地址', max_length=256, editable=False)
    trade_type = models.CharField(verbose_name=u'交易类型', max_length=16, editable=False)
    openid = models.CharField(verbose_name=u'用户标识(openId)', null=True, blank=True, max_length=128, editable=False)

    class Meta:
        verbose_name = u'微信统一订单'
        verbose_name_plural = u'微信统一订单'

    def __str__(self):
        return self.out_trade_no

    def _get_vlaue_dict(self):
        fieldsList = WeiXinOrder._meta.get_fields()
        return {item.attname: getattr(self, item.attname) for item in fieldsList if not item.auto_created and getattr(self, item.attname)}

    def to_xml(self):
        # sign data
        valueDict = self._get_vlaue_dict()
        valueDict['nonce_str'] = random_helper.generate_nonce_str(23)
        valueDict['sign'] = sign(valueDict)
        return xml_helper.dict_to_xml(valueDict)


class WeiXinPayResult(models.Model):
    order = models.OneToOneField(WeiXinOrder, on_delete=models.CASCADE, primary_key=True, editable=False, related_name='pay_result')
    prepayid = models.CharField(verbose_name=u'预支付交易会话标识', null=True, blank=True, max_length=64, db_index=True, editable=False)
    openid = models.CharField(verbose_name=u'用户标识(openId)', null=True, blank=True, max_length=128, editable=False)
    bank_type = models.CharField(verbose_name=u'付款银行', null=True, blank=True, max_length=16, editable=False)
    total_fee = models.SmallIntegerField(verbose_name=u'总金额', null=True, blank=True, editable=False)
    attach = models.CharField(verbose_name=u'商户附加数据', null=True, blank=True, max_length=128, editable=False)
    tradestate = models.CharField(verbose_name=u'交易状态', null=True, blank=True, max_length=32, editable=False)
    tradestatedesc = models.CharField(verbose_name=u'交易状态描述', null=True, blank=True, max_length=256, editable=False)

    def __str__(self):
        fieldsList = WeiXinPayResult._meta.get_fields()
        temp = []
        for field in fieldsList:
            if not field.auto_created:
                temp.append('{}:{}'.format(field.verbose_name, getattr(self, field.attname)))
        return ','.join(temp)



import logging
from django.http import HttpResponse
from openunipay.models import PAY_WAY_WEIXIN
from openunipay.paygateway import unipay

_logger = logging.getLogger('weixin_pay_notificaiton')


def process_notify(request):
    _logger.info('received weixin pay notification.body:{}'.format(request.body))
    unipay.process_notify(PAY_WAY_WEIXIN, request.body)
    return HttpResponse('<xml><return_code><![CDATA[SUCCESS]]></return_code><return_msg><![CDATA[OK]]></return_msg></xml>', 'application/xml', 200)


def process_qr_notify(request):
    _logger.info('received weixin qr pay notification.body:{}'.format(request.body))
    try:
        responseData = unipay.process_qr_pay_notify(PAY_WAY_WEIXIN, request.body)
        return HttpResponse(responseData, 'application/xml', 200)
    except:
        _logger.exception('process qr notify failed')
        return HttpResponse(None, 500)



class PayGateway(object):
    '''
    @summary: the base class for pay gateway
    '''

    def create_order(self, orderItemObj, clientIp, **kwargs):
        pass

    def query_order(self, orderNo):
        '''
        @summary: query pay result of order
        @return: PayResult
        '''
        pass

    def process_notify(self, requestContent):
        '''
        @summary: process notify from pay interface
        @return: PayResult
        '''
        pass

    def generate_qr_pay_url(self, productid):
        '''
        @summary: create url that can be used to generate qr code
        @return: url
        '''
        pass

    def process_qr_pay_notify(self, requestContent):
        '''
        @summary: process qr notify
        @return: proudct id,uid
        '''
        pass


class PayResult(object):
    def __init__(self, orderNo, succ=True, lapsed=False):
        self.orderno = orderNo
        self.succ = succ
        self.lapsed = lapsed

    @property
    def OrderNo(self):
        '''
        @summary: order No or merchant
        '''
        return self.orderno

    @property
    def Succ(self):
        '''
        @summary: True: paid successfully
        '''
        return self.succ

    @property
    def Lapsed(self):
        '''
        @summary: True: order is lapsed
        '''
        return self.lapsed



from django.conf import settings
from . import PayGateway
from django.db import transaction
from ..ali_pay.models import AliPayOrder
from ..ali_pay import ali_pay_lib

class AliPayGateway(PayGateway):
    
    @transaction.atomic
    def create_order(self, orderItemObj, clientIp):
        aliOrderObj = AliPayOrder()
        aliOrderObj.out_trade_no = orderItemObj.orderno
        aliOrderObj.subject = orderItemObj.product_desc
        aliOrderObj.body = orderItemObj.product_detail
        aliOrderObj.total_fee = orderItemObj.fee / 100
        aliOrderObj.it_b_pay = settings.ALIPAY['order_expire_in']
        aliOrderObj.save()
        ali_pay_lib.create_order(aliOrderObj)
        return aliOrderObj.compose_interface_data()
    
    @transaction.atomic
    def process_notify(self, requestContent):
        return ali_pay_lib.process_notify(requestContent)

    @transaction.atomic
    def query_order(self, orderNo):
        return ali_pay_lib.query_order(orderNo)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-22 00:39
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openunipay', '0005_product'),
    ]

    operations = [
        migrations.AddField(
            model_name='weixinorder',
            name='openid',
            field=models.CharField(blank=True, editable=False, max_length=128, null=True, verbose_name='用户标识(openId)'),
        ),
        migrations.AlterField(
            model_name='product',
            name='productid',
            field=models.CharField(max_length=50, primary_key=True, serialize=False, verbose_name='商品ID'),
        ),
    ]




class PayWayError(Exception):
    pass

class InsecureDataError(Exception):
    pass

class PayProcessError(Exception):
    
    def __init__(self, message):
        self.message = message
        
    def __str__(self, *args, **kwargs):
        return repr(self.message)



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-21 12:30
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openunipay', '0004_weixinqrpayentity_weixinqrpayrecord'),
    ]

    operations = [
        migrations.CreateModel(
            name='Product',
            fields=[
                ('productid', models.CharField(editable=False, max_length=50, primary_key=True, serialize=False, verbose_name='商品ID')),
                ('product_desc', models.CharField(max_length=128, verbose_name='商品描述')),
                ('product_detail', models.TextField(max_length=1000, verbose_name='商品详情')),
                ('fee', models.DecimalField(decimal_places=0, max_digits=6, verbose_name='金额(单位:分)')),
                ('weinxin_qrurl', models.CharField(blank=True, max_length=500, null=True, verbose_name='微信扫码支付URL')),
                ('date_create', models.DateTimeField(auto_now_add=True, verbose_name='创建时间')),
                ('date_update', models.DateTimeField(auto_now=True, verbose_name='修改时间')),
            ],
            options={
                'verbose_name_plural': '商品',
                'verbose_name': '商品',
            },
        ),
    ]



# -*- coding: utf-8 -*-
# Generated by Django 1.11.3 on 2017-07-21 00:41
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openunipay', '0003_auto_20170117_1106'),
    ]

    operations = [
        migrations.CreateModel(
            name='WeiXinQRPayEntity',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('appid', models.CharField(editable=False, max_length=32, verbose_name='公众账号ID')),
                ('mch_id', models.CharField(editable=False, max_length=32, verbose_name='商户号')),
                ('time_stamp', models.CharField(editable=False, max_length=10, verbose_name='时间戳')),
                ('product_id', models.CharField(editable=False, max_length=32, verbose_name='商品ID')),
            ],
            options={
                'verbose_name_plural': '微信扫码支付-二维码URL',
                'verbose_name': '微信扫码支付-二维码URL',
            },
        ),
        migrations.CreateModel(
            name='WeiXinQRPayRecord',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('appid', models.CharField(editable=False, max_length=32, verbose_name='公众账号ID')),
                ('mch_id', models.CharField(editable=False, max_length=32, verbose_name='商户号')),
                ('openid', models.CharField(editable=False, max_length=128, verbose_name='用户标识')),
                ('product_id', models.CharField(editable=False, max_length=32, verbose_name='商品ID')),
            ],
            options={
                'verbose_name_plural': '微信扫码支付-扫码纪录',
                'verbose_name': '微信扫码支付-扫码纪录',
            },
        ),
    ]



# -*- coding: utf-8 -*-
from django.contrib import admin
from .models import WeiXinOrder
from import_export import resources
from import_export.admin import ImportExportModelAdmin

class WeiXinOrderResource(resources.ModelResource):
    class Meta:
        model = WeiXinOrder
        import_id_fields = ('id',)

class WeiXinOrderAdmin(ImportExportModelAdmin):
    resource_class = WeiXinOrderResource
    # list page
    list_display = ('out_trade_no', 'body', 'attach', 'fee_type', 'total_fee', 'spbill_create_ip', 'time_start', 'time_expire', 'trade_type', 'get_pay_result')
    ordering = ('time_start',)
    search_fields = ['=out_trade_no', '=spbill_create_ip', ]
    
    def get_pay_result(self, obj):
        if hasattr(obj, 'pay_result'):
            return obj.pay_result
        else:
            return '-'
    get_pay_result.short_description = '支付结果'
    
    
admin.site.register(WeiXinOrder, WeiXinOrderAdmin)



# -*- coding: utf-8 -*-
from django.contrib import admin
from .ali_pay.admin import *
from .weixin_pay.admin import *

from .models import OrderItem, Product, PAY_WAY_WEIXIN
from openunipay.paygateway import unipay


class OrderItemResource(resources.ModelResource):
    class Meta:
        model = OrderItem
        import_id_fields = ('orderno', )


class OrderItemAdmin(ImportExportModelAdmin):
    resource_class = OrderItemResource
    # list page
    list_display = ('orderno', 'user', 'product_desc', 'payway', 'fee', 'dt_start', 'dt_end', 'dt_pay', 'paied')
    ordering = ('-dt_start', )
    search_fields = ['=orderno',
                     '=user', ]
    list_filter = ('payway',
                   'paied',
                   'product_desc', )

    def save_model(self, request, obj, form, change):
        if not obj.orderno:
            obj.initial_orlder()
        admin.ModelAdmin.save_model(self, request, obj, form, change)


########################### product #########################
class ProductResource(resources.ModelResource):
    class Meta:
        model = OrderItem
        import_id_fields = ('productid', )


class ProductAdmin(ImportExportModelAdmin):
    resource_class = ProductResource
    # list page
    list_display = ('productid',
                    'product_desc',
                    'fee',
                    'weinxin_qrurl',
                    'date_create',
                    'date_update', )
    ordering = ('-date_create', )
    search_fields = ['=productid', ]

    def save_model(self, request, obj, form, change):
        obj.weinxin_qrurl = unipay.generate_qr_pay_url(PAY_WAY_WEIXIN, obj.productid)
        admin.ModelAdmin.save_model(self, request, obj, form, change)


admin.site.register(OrderItem, OrderItemAdmin)
admin.site.register(Product, ProductAdmin)



import rsa
import base64
from django.conf import settings
from openunipay.ali_pay import logger

def sign(data):
    privateKey = _load_private_key(settings.ALIPAY['rsa_private_key_pem'])
    signBytes = rsa.sign(data.encode(), privateKey, 'SHA-1')
    signStr = str(base64.b64encode(signBytes), 'utf-8')
    return signStr

def verify(data, sign, pemKeyfile):
    sign = base64.b64decode(sign)
    pubKey = _load_public_key(pemKeyfile)
    result = False
    try:
        rsa.verify(data.encode(), sign, pubKey)
    except rsa.pkcs1.VerificationError:
        result = False
    else:
        result = True
    return result


def verify_ali_data(valueDict):
    logger.info('verifying data from ali')
    sign = valueDict['sign']
    # remove sign and sign_type
    del valueDict['sign']
    if 'sign_type' in valueDict:
        del valueDict['sign_type']
    # contact string need to verify
    temp = []
    for key in sorted(valueDict):
        if not valueDict[key]:
            continue
        temp.append('{}={}'.format(key, valueDict[key]))
    tempStr = '&'.join(temp)
    logger.info('string to verify:{}'.format(tempStr))
    return verify(tempStr, sign, settings.ALIPAY['ali_public_key_pem'])
    

def _load_private_key(pemKeyfile):
    with open(pemKeyfile) as keyFile:
        return rsa.PrivateKey.load_pkcs1(keyFile.read().encode('latin_1'))
    
def _load_public_key(pemKeyfile):
    with open(pemKeyfile) as keyFile:
        return rsa.PublicKey.load_pkcs1_openssl_pem(keyFile.read().encode('latin_1'))
    



from __future__ import absolute_import
from time import time
import datetime

def utc_now():
    return datetime.datetime.utcnow()

def local_now():
    return datetime.datetime.now()

def now_str():
    return local_now().strftime("%Y-%m-%d %H:%M:%S")

def get_timestamp():
    return int(time())

def get_unix_timestamp():
    return int((utc_now() - datetime.datetime(1970, 1, 1, tzinfo=None)).total_seconds())



# -*- coding: utf-8 -*-
# Generated by Django 1.10.5 on 2017-01-17 11:06
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('openunipay', '0002_remove_alipayorder_interface_data'),
    ]

    operations = [
        migrations.AlterField(
            model_name='weixinorder',
            name='total_fee',
            field=models.PositiveIntegerField(editable=False, verbose_name='总金额'),
        ),
    ]



import logging
from django.http import HttpResponse
from openunipay.models import PAY_WAY_ALI
from openunipay.paygateway import unipay

_logger = logging.getLogger('openunipay_ali_pay_notificaiton')

def process_notify(request):
    _logger.info('received ali pay notification.body:{}'.format(request.body))
    unipay.process_notify(PAY_WAY_ALI, request)
    return HttpResponse('success', 'text/plain-text', 200)



'''
Created on Nov 16, 2015

@author: panweif
'''
from django.core.management.base import BaseCommand
from django.contrib.auth.models import User

class Command(BaseCommand):
    help = 'create amdin superuser with password 123'

    def add_arguments(self, parser):
        pass

    def handle(self, *args, **options):
        User.objects.create_superuser('admin', '', '123')
        self.stdout.write('admin user created successfully')



'''
@summary: This is the main interface that can be used to create order, query order status and process notification 
@author: EricPan(pan.weifeng@live.cn)
@contact: EricPan(pan.weifeng@live.cn)
'''
from django.db import transaction
from openunipay.models import OrderItem, PAY_WAY_WEIXIN, PAY_WAY_ALI, Product
from openunipay.paygateway import weixin, alipay, PayResult
from openunipay import exceptions, logger
from openunipay.util import datetime

_PAY_GATEWAY = {PAY_WAY_WEIXIN: weixin.WeiXinPayGateway(),
                PAY_WAY_ALI: alipay.AliPayGateway(), }


@transaction.atomic
def create_order(orderno, payway, clientIp, product_desc, product_detail, fee, user=None, attach=None, expire=1440, **kwargs):
    '''
    @summary: create order
    @param orderno: order no
    @param payway: payway
    @param clientIp: IP address of the client that start the pay process
    @param product_desc: short description of the product
    @param product_detail: detail information of the product
    @param fee: price of the product. now, only support RMB. unit: Fen
    @param user: user identify
    @param attach: attach information. must be str
    @param expire: expire in minutes 
    '''
    if not is_supportted_payway(payway):
        raise exceptions.PayWayError()

    orderItemObj = OrderItem()
    orderItemObj.orderno = orderno
    orderItemObj.user = user
    orderItemObj.product_desc = product_desc
    orderItemObj.product_detail = product_detail
    orderItemObj.fee = fee
    orderItemObj.payway = payway
    orderItemObj.attach = attach
    orderItemObj.initial_orlder(expire)
    orderItemObj.save()

    # send order to pay gateway
    gatewayData = _PAY_GATEWAY[payway].create_order(orderItemObj, clientIp, **kwargs)
    logger.info('order created. orderno:{}, payway:{}, clientIp:{}, product:{},fee:{}, gateway data:{}'.format(orderno, payway, clientIp, product_desc, fee, gatewayData))
    return gatewayData


@transaction.atomic
def query_order(orderno):
    '''
    @summary: query status of order
    @param orderno: order no 
    '''
    orderItemObj = OrderItem.objects.get(orderno=orderno)
    if orderItemObj.paied:
        return PayResult(orderItemObj.orderno)
    elif orderItemObj.lapsed:
        return PayResult(orderItemObj.orderno, succ=False, lapsed=True)
    else:
        payResult = _PAY_GATEWAY[orderItemObj.payway].query_order(orderItemObj.orderno)
        _update_order_pay_result(payResult)
        return payResult


@transaction.atomic
def process_notify(payway, requestContent):
    '''
    @summary:  process async notification from pay interface
    @param payway: payway
    @param requestContent:  request body from pay interface
    @return: an instance of PayResult
    '''
    if not is_supportted_payway(payway):
        raise exceptions.PayWayError()
    payResult = _PAY_GATEWAY[payway].process_notify(requestContent)
    _update_order_pay_result(payResult)
    logger.info('process notify. payway:{}, content:{}'.format(payway, requestContent))
    return payResult


def is_supportted_payway(payway):
    return payway in _PAY_GATEWAY


def _update_order_pay_result(payResult):
    if payResult.Succ:
        orderItemObj = OrderItem.objects.get(orderno=payResult.OrderNo)
        orderItemObj.paied = True
        orderItemObj.lapsed = False
        orderItemObj.dt_pay = datetime.local_now()
        orderItemObj.save()
    elif payResult.Lapsed:
        orderItemObj = OrderItem.objects.get(orderno=payResult.OrderNo)
        orderItemObj.paied = False
        orderItemObj.lapsed = True
        orderItemObj.dt_pay = None
        orderItemObj.save()


@transaction.atomic
def generate_qr_pay_url(payway, productid):
    '''
    @summary: create qr pay url
    @param productid: product id
    '''
    if not is_supportted_payway(payway):
        raise exceptions.PayWayError()

# send order to pay gateway
    url = _PAY_GATEWAY[payway].generate_qr_pay_url(productid)
    logger.info('qr pay url generated. productid:{}, url:{}'.format(productid, url))
    return url


@transaction.atomic
def process_qr_pay_notify(payway, requestContent):
    if not is_supportted_payway(payway):
        raise exceptions.PayWayError()
    return _PAY_GATEWAY[payway].process_qr_pay_notify(requestContent)
